<?php
/*
 * Permet de localiser plusieurs adresses IP avec IP2LOCATION
 *
 * Peut être "include" dans un script PHP. Appeler la fonction getLocations()
 * Peut répondre à une requête HTTP, methode GET paramètre ip ( 1 seule adresse IP )
 *
 * */
const IP2LOCATION_ROOT = __DIR__ . '/IP2Location-PHP-Module/';
const IP2LOCATION_DB_PATTERN = '@DB(\d+)(?:\.IPV(\d+))?\.BIN$@i';

if(is_dir(IP2LOCATION_ROOT . '/databases')) {

	$dbs = glob(IP2LOCATION_ROOT . '/databases/IP2LOCATION*.BIN');
	if(count($dbs) > 1) {
		# On trie pour avoir la base de données la plus détaillée au début, avec préférence pour IP-v6
		usort($dbs, function($a, $b) {
			preg_match(IP2LOCATION_DB_PATTERN, $a, $m); $i = intval($m[1]);
			preg_match(IP2LOCATION_DB_PATTERN, $b, $n); $j = intval($n[1]);
			if($i != $j) {
				return $j - $i;
			}
			if(empty($m[2])) { return  1; }
			if(empty($n[2])) { return -1; }
			return (intval($n[2]) - intval($m[2]));
		});
	}

	define('IP2LOCATION_DB', $dbs[0]);

	require_once IP2LOCATION_ROOT . 'IP2Location.php';

	// Optimisation pour IP2LOCATION
	define('IP2LOCATION_DB_FIELDS', array(
		1 => array(\IP2Location\Database::COUNTRY),
		3 => array(\IP2Location\Database::COUNTRY, \IP2Location\Database::REGION_NAME, \IP2Location\Database::CITY_NAME),
		5 => array(\IP2Location\Database::COUNTRY, \IP2Location\Database::REGION_NAME, \IP2Location\Database::CITY_NAME, \IP2Location\Database::COORDINATES),
		9 => array(\IP2Location\Database::COUNTRY, \IP2Location\Database::REGION_NAME, \IP2Location\Database::CITY_NAME, \IP2Location\Database::COORDINATES, \IP2Location\Database::ZIP_CODE),
		11 => array(\IP2Location\Database::COUNTRY, \IP2Location\Database::REGION_NAME, \IP2Location\Database::CITY_NAME, \IP2Location\Database::COORDINATES, \IP2Location\Database::ZIP_CODE, \IP2Location\Database::TIME_ZONE),
	));

	/*
	 * Localise plusieurs adresses IP à l'aide de la base de données publiée par https://ip2location.com
	 *
	 * @param $request tableau d'adresses IP
	 * @return tableau indexé par adresses IP. retourne
	 *     nom et code pays,
	 *     ville et région,
	 *     latitude/longitude
	 *     code postal et fuseau horaire
	 *     selon base de données utilisée, disponible gratuitement ou payante à :
	 *     https://lite.ip2location.com/ip2location-lite
	 *
	 * */
	function getLocations($request) {
		if(!is_array($request)) {
			return false;
		}

		$result = array();

		// To speed up change FILE_IO to SHARED_MEMORY or MEMORY_CACHE. Check for enough memory
		// $db = new \IP2Location\Database(IP2LOCATION_DB, \IP2Location\Database::FILE_IO);
		$db = new \IP2Location\Database(IP2LOCATION_DB);
		if($db) {
			$dbType = $db->getType();
			$fields = (isset(IP2LOCATION_DB_FIELDS[$dbType])) ? IP2LOCATION_DB_FIELDS[$dbType] : $dbType = $db->getFields();
			foreach($request as $ip) {
				$ip1 = filter_var($ip, FILTER_VALIDATE_IP, FILTER_FLAG_NO_PRIV_RANGE | FILTER_FLAG_NO_RES_RANGE);
				if(!empty($ip1)) {
					$record = $db->lookup($ip1, $fields);
					if(count($request) == 1) {
						$record['cidr'] = $db->get_cidr($ip);
					}
					$result[$ip] = $record;
				}
			}
		}

		return $result;
	}

	if(isset($_SERVER['SCRIPT_NAME']) and basename($_SERVER['SCRIPT_NAME']) == basename(__FILE__)) {
		# Requête HTTP
		$ip = filter_input(INPUT_GET, 'ip', FILTER_VALIDATE_IP);
		if(!empty($ip)) {
			$records = getLocations(array($ip));
			header('Content-Type: application/json');
			header('Cache-Control: No cache');
			echo json_encode($records);
		}
	}
} else {
	if(isset($_SERVER['SCRIPT_NAME']) and basename($_SERVER['SCRIPT_NAME']) == basename(__FILE__)) {
	}
}
