<?php

/*
 * Extrait les intervalles d'adresses IP de la base de données IP2LOCATION
 * pour un ou plusieurs code de pays séparés par des espaces
 * */
const IP2LOCATION_ROOT = '../IP2Location-PHP-Module/';
const IP2LOCATION_DBNAME_PATH = IP2LOCATION_ROOT . 'databases/IP2LOCATION-LITE-DB1.BIN';

class MyDatabase {
	// public const FILE_IO = 100001;
	const EXCEPTION_FILE_OPEN_FAILED = 10008;
	const COLUMNS = [8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8];
	const DEFAULT_COUNTRIES = 'FR BE CH CA';

	/**
	 * The machine's float size.
	 */
	private static $floatSize = null;
	private $columnWidth = [];

	public function __construct() {

		$rfile = IP2LOCATION_DBNAME_PATH;
		$size = filesize($rfile);
		// $this->mode = self::FILE_IO;
		$this->resource = @fopen($rfile, 'r');
		if ($this->resource === false) {
			throw new \Exception(__CLASS__ . ": Unable to open file '{$rfile}'.", self::EXCEPTION_FILE_OPEN_FAILED);
		}
		if (self::$floatSize === null) {
			self::$floatSize = \strlen(pack('f', M_PI));
		}

		// extract database metadata
		$this->type = $this->readByte(1) - 1;
		$this->columnWidth[4] = $this->readByte(2) * 4;
		$this->columnWidth[6] = $this->columnWidth[4] + 12;

		$this->offset = array(
			4 => -4,
			6 =>  8,
		);

		$this->year = 2000 + $this->readByte(3);
		$this->month = $this->readByte(4);
		$this->day = $this->readByte(5);
		$this->date = date('Y-m-d', strtotime("{$this->year}-{$this->month}-{$this->day}"));

		$this->ipCount[4] = $this->readWord(6);
		$this->ipBase[4] = $this->readWord(10);		//hjlim readword
		$this->ipCount[6] = $this->readWord(14);
		$this->ipBase[6] = $this->readWord(18);
		$this->indexBaseAddr[4] = $this->readWord(22);		//hjlim
		$this->indexBaseAddr[6] = $this->readWord(26);		//hjlim
	}

	public function getIpsCountry($codeCountries=self::DEFAULT_COUNTRIES, $ipVersion=4) {
		if(empty(trim($codeCountries))) {
			$codeCountries = self::DEFAULT_COUNTRIES;
		}
		$isoCodes = array_map('trim', explode(' ', strtoupper(trim($codeCountries))));
		if(empty($isoCodes)) { return false; }
		$pattern = (count($isoCodes) > 1) ? '@^(?:' . implode('|', $isoCodes) .  ')$@' : '@' . $isoCodes[0] . '@';

		$base = $this->ipBase[$ipVersion];
		$offset = $this->offset[$ipVersion];
		$width = $this->columnWidth[$ipVersion];

		//hjlim
		$indexBaseStart = $this->indexBaseAddr[$ipVersion];

		if($indexBaseStart > 0) {
			$ipNumber = 16777216;
			switch ($ipVersion) {
			case 4:
				$ipNum1_2 = (int) ($ipNumber / 65536);
				$indexPos = $indexBaseStart + ($ipNum1_2 << 3);
				$rawOffset = $this->columnWidth[4] + 4;
				break;

			case 6:
				$ipNum1 = (int) (bcdiv($ipNumber, bcpow('2', '112')));
				$indexPos = $indexBaseStart + ($ipNum1 << 3);
				$rawOffset = $this->columnWidth[6];
				break;

			default:
				return false;
			}
		}

		$result = array();
		for($low=0, $high=$this->ipCount[$ipVersion]; $low<$high; $low++) {
			$pointer = $base + $offset + $low * $width;
			$this->rawPositionsRow = $this->read($pointer - 1, $rawOffset);
			$countryCode = $this->readString(self::COLUMNS[$this->type]);
			if(preg_match($pattern, $countryCode)) {
				// Read IP ranges to get boundaries
				$result[] = array(
					$this->readIp($ipVersion, $base + $width * $low), // $ip_from
					$this->readIp($ipVersion, $base + $width * ($low + 1)) - 1, // $ip_to - 1
					$countryCode,
				);
			}
		}

		return $result;
	}

	private function read($pos, $len)
	{
		fseek($this->resource, $pos, SEEK_SET);
		return fread($this->resource, $len);
	}

	private function readString($pos, $additional = 0)
	{
		// Get the actual pointer to the string's head by extract from the raw row
		$spos = unpack('V', substr($this->rawPositionsRow, $pos, 4))[1] + $additional;

		// Read as much as the length (first "string" byte) indicates
		return $this->read($spos + 1, $this->readByte($spos + 1));
	}

	private function readIp($ipVersion, $pos)
	{
		if ($ipVersion === 4) {
			// Read a standard PHP int
			return self::wrap32($this->readWord($pos));
		} elseif ($ipVersion === 6) {
			// Read as BCMath int (quad)
			return $this->readQuad($pos);
		}
		// unrecognized
		return false;
	}

	private function readByte($pos)
	{
		// Unpack a byte's worth of data
		return self::wrap8(unpack('C', $this->read($pos - 1, 1))[1]);
	}

	private function readWord($pos)
	{
		// Unpack a long's worth of data
		return self::wrap32(unpack('V', $this->read($pos - 1, 4))[1]);
	}

	private function readQuad($pos)
	{
		// Use BCMath ints to get a quad's (128-bit) value
		return self::bcBin2Dec($this->read($pos - 1, 16));
	}

	private static function wrap8($x)
	{
		return $x + ($x < 0 ? 256 : 0);
	}

	private static function wrap32($x)
	{
		return $x + ($x < 0 ? 4294967296 : 0);
	}

}
